#!/usr/bin/perl -w
use strict;

use CGI;
use POSIX;
use File::Path;
use File::Copy;
use File::stat;
use FindBin;

my $stype = "unix";
my $document_root = $ENV{DOCUMENT_ROOT} || "";
my $server_name = $ENV{SERVER_NAME} || "";
my $server_port = $ENV{SERVER_PORT} || "";
my $public_dir = $document_root."/public";
my $private_dir = $document_root."/../private";
my $url_path = "http://".$server_name
                .(($server_port != 80)? (':'.$server_port) : '' )
                .'/public';
my $action = "configure";
my $path_delim = "/";
my $source_dir = $FindBin::Bin;
my $mode = 0666;

my $query = new CGI;
my @errors;

&main_install();

############################################################################
sub print_errors
############################################################################
{
    print "<h3>Error details:</h3><p>\n";
    print join("<br>\n", @errors);
}   ##print_errors

############################################################################
sub clear_errors
############################################################################
{
    @errors = ();
}   ##clear_errors

############################################################################
sub add_error       #19.02.03 13:01
############################################################################
{
    $errors[++$#errors] = $_[0];
}   ##add_error

# Guess systems depends
sub guess
{
    #print "$^O\n";
    #print $^O =~ /Win/, "\n";
    my $server_software = $ENV{SERVER_SOFTWARE} || "";
    if ($server_software =~ /Microsoft-IIS/ || $^O =~ /Win/) {
		$stype = "win";
        $path_delim = "\\";
        $public_dir = $document_root."\\public";
        $private_dir = $document_root."\\..\\private";
	};
    if ( -f $source_dir.$path_delim.'env.pl' ) {
        eval {
            require $source_dir.$path_delim.'env.pl';
            $stype = $env::System_type;
            $path_delim = $env::Path_Delim;
            $public_dir = $env::Public_base;
            $private_dir = $env::Private_base;
        }
    }
}

sub get_params
{
    $action = $query->param('action') || 'configure';

    if ($action ne 'configure') {
		$stype = $query->param('stype');
#		$user = $query->param('user');
#		$group = $query->param('group');
		$public_dir = $query->param('public_dir');
		$private_dir = $query->param('private_dir');
		$url_path = $query->param('url_path');
	}

    if ( $stype eq 'win' ) {
        $path_delim = "\\";
    } else {
        $path_delim = "/";
    }

#	($stype eq 'unix' && $user && $group)
	if ( $stype && $public_dir && $private_dir && $url_path ) {
		return "Ok";
	} else {
		return "";
	}
}

sub test_perl_version
{
    my $version = 5.005;
	unless (eval "require $version") {
        add_error("Perl version must bee $version or above ");
        return 0;
	}
    return 1;
}

############################################################################
sub check_folder
############################################################################
 {
    my $name = 'testcanwrite';
    my $len = length ($name);
    my $folder = $_[0];
    my $file = $folder.$path_delim.$name;
    my $success = 1;
    my $created = 0;
    open (FD, "> $file") or $success = 0;
    $created = $success;
    if ($success && syswrite (FD,$name,$len) != $len) {
           $success = 0;
    }
    if ( $created ) {
        close(FD);
        unlink($file);
    }
    if ( !$success ) {
        add_error("Folder does not exists or no write permissions in folder: $folder");
    }
    return $success;
}   ##check_folder

############################################################################
sub check_folders
############################################################################
{
    my $cs = check_folder($source_dir);
    my $cu = check_folder($public_dir);
    my $cr = check_folder($private_dir);
    return $cs && $cu && $cr;
}   ##check_folders

############################################################################
sub check_folders2
# check in folders after install
############################################################################
{
    check_folder($public_dir.$path_delim."packages") or return 0;
    check_folder($private_dir.$path_delim."packages") or return 0;
    check_folder($private_dir.$path_delim."import") or return 0;
    check_folder($private_dir.$path_delim."templates") or return 0;
    check_folder($private_dir.$path_delim."lib") or return 0;
    return 1;
}   ##check_folders2

############################################################################
sub copyFolder      #11.02.03 16:10
############################################################################
{
    my ($folder, $base_from, $base_to, $mod, $preserve, @dummy) = @_;
    my $fmask = '<'.$base_from.$path_delim.$folder.$path_delim.'*>';
    $fmask =~ s/\\/\\\\/g;
    my @files = eval $fmask;
    #print "Copy folder $folder from $base_from to $base_to","<br>\n";
    #print $fmask,"<br>\n";
    #print "@files","<br>\n";
    my $dfolder = $base_to.$path_delim.$folder;
    if ( ! -d $dfolder ) {
        File::Path::mkpath($dfolder, 0, 0770);
        if ($stype eq 'unix') {
            my $modplus = $mod;
            $modplus |= (($modplus >> 2)&0111);
            if (!chmod $modplus, $base_to.$path_delim.$folder) {
                add_error("Can not chmod 0770 folder $$base_to.$path_delim.$folder");
                return 0;
            }
        }
    } else {
        if ($stype eq 'unix') {
            my $modplus = $mod;
            $modplus |= (($modplus >> 2)&0111);
            chmod $modplus, $base_to.$path_delim.$folder;
            # no diags
        }
    }
    foreach my $file (@files) {
        my ($name, $path, $type) = File::Basename::fileparse($file, '');
        my $dfile = $base_to.$path_delim.$folder.$path_delim.$name.$type;
        if ( -f $file && !-d $file && !( $preserve && -f $dfile )) {
            #print "Copy from $file to $dfile<br>\n";
            if (!File::Copy::syscopy($file, $dfile)) {
                add_error("Can not open file for writing: $dfile");
                return 0;
            }
            if ($stype eq "unix") {
               my $modplus = $mod;
               if ( $dfile =~ /.*\.pl/ ) {
                  $modplus |= (($modplus >> 2)&0111);
               }
               my $so = sprintf ('%4o', $modplus);
               #print "Doing chmod $so file $dfile<br>\n";
               if (!chmod $modplus, $dfile) {
                   add_error("Can not chmod $so file $dfile");
                   return 0;
               }
            }
        } else {
            # chmod without diags
            if ($stype eq "unix") {
               my $modplus = $mod;
               if ( $dfile =~ /.*\.pl/ ) {
                  $modplus |= (($modplus >> 2)&0111);
               }
               chmod $modplus, $dfile;
            }
        }
    }
    return 1;
}   ##copyFolder

############################################################################
sub copyDist        #12.02.03 15:11
############################################################################
{
    copyFolder('.', $source_dir.$path_delim.'private', $private_dir, $mode, 1) or return 0;
    copyFolder('bin', $source_dir.$path_delim.'private', $private_dir, $mode, 0) or return 0;
    copyFolder('import', $source_dir.$path_delim.'private', $private_dir, $mode, 0) or return 0; # empty
    copyFolder('packages', $source_dir.$path_delim.'private', $private_dir, $mode, 0) or return 0; # empty
    copyFolder('templates', $source_dir.$path_delim.'private', $private_dir, $mode, 1) or return 0;
    copyFolder('lib', $source_dir.$path_delim.'private', $private_dir, $mode, 0) or return 0; # empty
    copyFolder('packages_bak', $source_dir.$path_delim.'private', $private_dir, $mode, 0) or return 0; # empty

    copyFolder('.', $source_dir.$path_delim.'public', $public_dir, $mode, 0) or return 0;
    copyFolder('stats', $source_dir.$path_delim.'public', $public_dir, $mode, 0) or return 0;
    copyFolder('packages', $source_dir.$path_delim.'public', $public_dir, $mode, 0) or return 0; # empty
    return 1;
}   ##copyDist

############################################################################
sub guess_mode
############################################################################
{
    my $name = 'testmode';
    my $len = length ($name);
    my $folder = $_[0];
    my $file = $folder.$path_delim.$name;
    my $rmode = 0666;
    my $created = 1;
    open (FD, "> $file") or $created = 0;
    if ( $created ) {
        close(FD);
        my $fs = stat($file);
        my $ds = stat($folder);
        if ( defined ($fs) && defined($ds) ) {
            if ( $fs->uid() == $ds->uid() || $fs->gid() == $ds->uid() ||
                 $fs->uid() == $ds->gid() || $fs->gid() == $ds->gid() ) {
                $rmode = 0660;
            }
        }
        unlink($file);
    }
    return $rmode;
}   ##guess_mode

############################################################################
sub main_install
############################################################################
{
	my $all_values_ok;
    print <<HEADER;
Content-Type: text/html

<html>
<head><title>Install Anti-Leecj srcipts v. 2.0</title></head>
<body>
HEADER

	guess();
	$all_values_ok = get_params();
    if ($action eq 'configure') {
        configure_exec();
    }
    else  {
        if (!$all_values_ok) {
                print "<div align=center> \n";
                print "<h4> Error: You need define all values </h4>";
                print "</div> <p>\n";
                configure_exec();
        } else {
            install_exec();
        }
    }

    print <<FOOTER;
</body>
</html>
FOOTER
}

############################################################################
sub configure_exec
############################################################################
{
    if ( !test_perl_version() ) {
        print <<HTML;
        <h3>Warning</h3>
        Incompatile Perl version. You may proceed further at your own risk.
HTML
        print_errors();
    }
    configure_html();
}   ##configure_exec

############################################################################
sub install_exec_inner
############################################################################
{
    check_folders() or return 0;
    if ( $stype == 'unix' ) {
        $mode = guess_mode($private_dir);
    }
    copyDist() or return 0;
    write_env($source_dir) or return 0;
    write_env($public_dir) or return 0;
    write_env($public_dir.$path_delim."stats") or return 0;
    write_env($private_dir.$path_delim."bin") or return 0;

    check_folders2() or return 0;
    return 1;
}   ##install_exec_inner

############################################################################
sub install_exec
############################################################################
{
    if ( !install_exec_inner() ) {
        notify_failure_html();
    } else {
        congratulations_html();
    }
}   ##install_exec

sub configure_html
{
	my $unixchk = "";
	my $winchk = "";

	if ($stype eq "win") {
        $winchk = "selected";
	} else {
        $unixchk = "selected";
	}

	print <<HTML;
	<table align=center>
    <form method=POST action=install.pl>

	<tr>
	 <td colspan=2 align=center><h3> First you need to Configure </h3> </td>
	</tr>

	<tr>
	 <td> System type is </td>
	 <td>
	   <select name=stype>
		<option value=unix $unixchk> Unix </option>
		<option value=win  $winchk> Windows </option>
	   </select>
	 </td>
	</tr>
	<tr>
     <td><em>Public</em> directory to install to</td>
	 <td>
	   <input type=text name=public_dir size=40 value='$public_dir'>
	 </td>
	</tr>
	<tr>
     <td><em>Private</em> directory to install to</td>
	 <td>
	   <input type=text name=private_dir size=40 value='$private_dir'>
	 </td>
	</tr>
	<tr>
     <td>Url Path to <em>Public</em> directory</td>
	 <td>
	   <input type=text name=url_path size=40 value='$url_path'>
	 </td>
	</tr>
	<tr>
	 <td>
	   <p><p><input type=submit value='Continue'>
	   <input type=hidden name='action' value='install'>
	 </td>
	 <td> </td>
	</tr>

	</form>
	</table>
HTML

}

############################################################################
sub congratulations_html
############################################################################
{
    print <<CONGR;
    <table align=center border=1><tr><td align=center>
    <h3>Congratulations!</h3>
    AL Download Protection is successfully transferred to the host.<br>
    </td></tr><tr><td align=center>
    Installer will inspect the host for required Perl packages
    and instruct your to install one's in question.<br>
    Press <a href=check_perl.pl target=_blank><b>HERE</b></a>
    to inspect the host.
    </td></tr><tr><td align=center>
    You need to setup login and password to get access to management of
    AL Download Protection. Press <a href=new_account.pl target=_blank><b>HERE</b></a>
    to setup. (This may not work until all packages are installed.)
    </td></tr><tr><td align=center>
    To test the installed software, log in
    <a target=_blank href=$url_path><b>HERE</b></a>.
    </td></tr></table>
CONGR
}   ##congratulations

############################################################################
sub notify_failure_html
############################################################################
{
    print <<FAIL;
    <h3>Installation aborted due to errors.</h3>
    Please, correct the errors and try once more.<br>
    Press <b>Refresh</b> to run installation again with the same settings.<br>
    Press <b>Back</b> to change the settings.<br>
    Contact <a href=mailto:support\@anti-leech.com>support\@anti-leech.com</a>
    if you can not solve the problems by yourself
    and for other assistance.
FAIL
    print_errors();
}   ##notify_failure

sub write_env
{
#    my $dir_path = $_[0] || "";
#    my $path = $public_dir.$path_delim.$dir_path.$path_delim."env.pl";
     my $path = $_[0].$path_delim."env.pl";
     my $path_delim2 = $path_delim;
     $path_delim2 =~ s/\\/\\\\/g;
     my $mode2 = sprintf('%04o', $mode);

    #print "Writing envetory file to $path<p>";
    if ( !open (FD, "> $path")) {
        add_error("Can not open file for writing: $path");
        return 0;
    }

print FD <<ENV_PL;
package env;
BEGIN {
 \$Path_Delim   = \'$path_delim2\';
 \$Public_base  = \'$public_dir\';
 \$Private_base = \'$private_dir\';
 \$Server_Root  = \'$url_path\';
 \$System_type  = \'$stype\';
 \$File_mode    = $mode2;

###################################
# Const section	#
###################################
 \$Template_base= \$Private_base.\$Path_Delim.\"templates\";
 \$Package_base = \$Private_base.\$Path_Delim.\"packages\";
 \$Backup_base =  \$Private_base.\$Path_Delim.\"packages_bak\";
 \$account      = \$Private_base.\$Path_Delim.\"packages\";
 \$Import_base  = \$Private_base.\$Path_Delim.\"import\";
 \$bin          = \$Private_base.\$Path_Delim.\"bin\";
 \$lib          = \$Private_base.\$Path_Delim.\"lib\";

 \$infile       = \$Private_base.\$Path_Delim.\"passwd.txt\";
 \$statfile     = \$Private_base.\$Path_Delim.\"statistics_daily\";
 \$statlog      = \$Private_base.\$Path_Delim.\"statistics_log\";
 \$package_crypt= \$Private_base.\$Path_Delim.\"package_crypt\";
 \$package_referer= \$Private_base.\$Path_Delim.\"package_referer\";
 \$recrypt_lock = \$Private_base.\$Path_Delim.\"recrypt_lock\";
 \$recrypt_lock2 = \$Private_base.\$Path_Delim.\"recrypt_lock2\";
 \$stop_daemon   = \$Private_base.\$Path_Delim.\"stop_daemon";

 \$template_xml = \$Template_base.\$Path_Delim.\"template.xml\";
 \$template_html = \$Template_base.\$Path_Delim.\"template.html\";

 \$extension    =  \".alp\";
 \$extension_html  =  \".html\";
 \$extension_crypt =  \".alp\";

 \$Package_href_base = \$Server_Root.\"/packages/\";
 \$Redirect_path= \$Server_Root.\"/admin.pl\";
 \$Css_file_path= \$Server_Root.\"/pac_manager.css\";
 \$LineGif_file_path= \$Server_Root.\"/line.gif\";
}
use lib (\$bin, \$lib);
1;
ENV_PL
    close(FD);
	if ($stype eq "unix") {
        if (!chmod $mode, $path) {
            add_error("Can not chmod on $_");
            return 0;
        }
	}
    return 1;
}


