#!/usr/bin/perl -w
use strict;
use FindBin;
my $source_dir = $FindBin::Bin;

require $source_dir.'/list_modules.pl';

my $System_type = '';
my $Private_base = '';
my $lib = '';
my $expat_home = '';
my $expat_inc = '';
my $expat_lib = '';
my $pref = '';

if ( -f $source_dir.'/env.pl' ) {
    require $source_dir.'/env.pl';
    $System_type = $env::System_type;
    $Private_base = $env::Private_base;
    $pref = $Private_base;
    $lib = $env::lib;
    $expat_home = $Private_base.'/expat';
    $expat_inc = $expat_home.'/include';
    $expat_lib = $expat_home.'/lib';
}

sub inst_perl;
sub inst_libwww;
sub inst_expat;
sub inst_xmlpars;
sub inst_xmldom;

my @distribs = (
# name                  dist_mods                             stat; deps; inst
['Digest-MD5-2.20',     ['Digest::MD5'],                        '', [], \&inst_perl],
['MIME-Base64-2.12',    ['MIME::Base64'],                       '', [], \&inst_perl],
['IO-1.20',             ['IO::Socket', 'IO::Dir'],              '', [], \&inst_perl],
['URI-1.22',            ['URI::URL'],                           '', [], \&inst_perl],
['libwww-perl-5.69',    ['LWP::UserAgent'],                     '', [], \&inst_libwww],
['XML-RegExp-0.03',     ['XML::RegExp'],                        '', [], \&inst_perl],
['expat-1.95.5',        [],                                     '', [], \&inst_expat],
['XML-Parser-2.31',     ['XML::Parser', 'XML::Parser::Expat'],  '', ['expat-1.95.5'], \&inst_xmlpars],
['XML-DOM-1.42',        ['XML::DOM'],                           '', [], \&inst_xmldom]
);

select STDOUT;
$| = 1;
open (STDERR, ">& STDOUT");

print <<HEADER;
Status: 200 OK
Content-Type: text/html

<html>
<head>
    <title>Installing Perl modules</title>
</head>
<body>
<h2>Installing Perl modules</h2>
HEADER

# Check we are invoked in correct environment
if ( $lib eq '') {
    print <<HTML;
    <p><b>Error:</b> you should invoke this script after successfull completion
    of installation of AL Download Protection itself.
HTML
    print_footer();
    exit;
}
#print "$System_type\n";
if ( 0 && $System_type ne 'unix') {
    print <<HTML;
    <p><b>Error:</b> this script is intended for usage
    on Unix machines only.
HTML
    print_footer();
    exit;
}

#finding archive and unpacking it
if ( -f $source_dir.'/required-unix.tar.gz' &&
     ! -f $source_dir.'/.untar_done')
{
    print <<HTML;
<p>
Installer have found an archive <i>required-unix.tar.gz</i>
and will decompress it.
</p>
    <hr>
    <pre>
HTML
    run_shell(<< "COMMANDS");
cd $source_dir
tar -zxf required-unix.tar.gz && touch .untar_done
chmod -R a+w . 2>/dev/null
COMMANDS
  # the last command - to make delete easier
    print <<HTML;
    </pre>
HTML
}

#converting module list into hash
my %hmodules = ();
foreach my $mod (@list_modules::modules) {
    $hmodules{$mod} =
        ( list_modules::check_module($mod) )?
            'present' : 'absent';
}

#foreach my $k (keys %hmodules) {
#    print "$k $hmodules{$k} <br>\n";
#}

# enumerate distributions, evaluate and list them
print <<HTML;
<p>
Installer will search for Perl modules
required by AL Download Protection and for corresponding
distrubutions.
</p>
<table border=1>
<tr><td>Distribution</td><td>Status</td><td>Contains modules</td><td>Module status</td></tr>
HTML

# Pass One
foreach my $dist (@distribs) {
    my $dist_name = \$dist->[0];
    my $dist_mods =  $dist->[1];
    my $dist_stat = \$dist->[2];
    my $dist_deps =  $dist->[3];
    my $dist_inst =  $dist->[4];
    $$dist_stat = 'skipped';
    #print "$$dist_name\n";
    #print "@{$dist_mods}\n";
    foreach my $mod (@{$dist_mods}) {
        if ( exists $hmodules{$mod} ) {
            if ( $hmodules{$mod} eq 'absent' ) {
                $$dist_stat = 'needed';
            }
        }
    }
    if ( $$dist_stat eq 'needed' ) {
        #print 'Checkin deps: ',join(',',@{$dist_deps}),"<br>\n";
        foreach my $dep (@{$dist_deps}) {
            foreach my $dist2 (@distribs) {
                my $dist2_name = $dist2->[0];
                my $dist2_stat = \$dist2 ->[2];
                #print " Comparing: $dep vs $dist2_name <br>\n";
                if ( $dist2_name eq $dep ) {
                    #print "Comparison success<br>\n";
                    $$dist2_stat = 'needed';
                }
            }
        }
    }
}
# Pass Two
foreach my $dist (@distribs) {
    my $dist_name = \$dist->[0];
    my $dist_mods =  $dist->[1];
    my $dist_stat = \$dist->[2];
    my $dist_deps =  $dist->[3];
    my $dist_inst =  $dist->[4];
    #print "$$dist_name\n";
    #print "@{$dist_mods}\n";
    my $span = @{$dist_mods}; #count
    if ( $$dist_stat eq 'needed' ) {
        if ( -d $source_dir.'/'.$$dist_name ) {
           $$dist_stat = 'found';
        }
        else  {
           $$dist_stat = 'NOT FOUND';
        }
    }
    print <<HTML;
    <tr><td rowspan=$span valign=top>$$dist_name</td><td rowspan=$span valign=top>$$dist_stat</td>
HTML
    foreach my $mod (@{$dist_mods}) {
        my $mod_stat = $hmodules{$mod} || 'not needed';
        if ( $mod ne $dist_mods->[0] ) {
            print "<tr>";
        }
        print <<HTML;
        <td>$mod</td><td>$mod_stat</td></tr>
HTML
    }
    print <<HTML;
    </tr>
HTML
}

print <<HTML;
</table>
<p>
Installer will install selected distributions
required by AL Download Protection.
</p>
HTML

# entering main loop: installing distribs
foreach my $dist (@distribs) {
    my $dist_name = \$dist->[0];
    my $dist_mods =  $dist->[1];
    my $dist_stat = \$dist->[2];
    my $dist_deps =  $dist->[3];
    my $dist_inst =  $dist->[4];
    if ( $$dist_stat ne 'found' ) {next;}

    print <<HTML;
    <h2>Installing distribution $$dist_name</h2>
    <hr>
    <pre>
HTML
    my $result = 0;
    if ( !check_folder($source_dir.'/'.$$dist_name) ) {
        print "Can't start: need permissions for write to directory ${$dist_name}";
    } else {
        $result = &$dist_inst($$dist_name) == 0;
    }
    $$dist_stat = $result? 'installed' : 'failed';
    my $comment = $result?
       "Distribution $$dist_name installed successfully":
       "Distribution $$dist_name - installation failed";
    print <<HTML;
    </pre>
    <hr>
    <h3>$comment</h3>
HTML
}

    print <<HTML;
<p>To remove distributions easier...</p>
    <hr>
    <pre>
HTML
    run_shell(<< "COMMANDS");
cd $source_dir
chmod -R a+w ./* 2>/dev/null
cd $lib
chmod -R a+w ./* 2>/dev/null
COMMANDS
    print <<HTML;
    </pre>
    <hr>
HTML

print <<HTML;
<h2>Module installation summary</h2>
<table border=1>
<tr><td>Distribution</td><td>Status</td></tr>
HTML

# print simmary
foreach my $dist (@distribs) {
    my $dist_name = \$dist->[0];
    my $dist_mods =  $dist->[1];
    my $dist_stat = \$dist->[2];
    print <<HTML;
    <tr><td>$$dist_name</td><td>$$dist_stat</td></tr>
HTML
}

print <<HTML;
</table>

<p>Please, check presense of required modules again,
invoke: <a href=check_perl.pl>check_perl.pl</a>.
HTML

exit;

print_footer();

############################################################################
sub print_footer
############################################################################
{
    print <<FOOTER;
</body>
</html>
FOOTER
}   ##print_footer

############################################################################
sub check_folder
############################################################################
 {
    my $name = 'testcanwrite';
    my $len = length ($name);
    my $folder = $_[0];
    my $file = $folder.$env::Path_Delim.$name;
    my $success = 1;
    my $created = 0;
    open (FD, "> $file") or $success = 0;
    $created = $success;
    if ($success && syswrite (FD,$name,$len) != $len) {
           $success = 0;
    }
    if ( $created ) {
        close(FD);
        unlink($file);
    }
    return $success;
}   ##check_folder

############################################################################
sub run_shell
############################################################################
{
    my $commands = $_[0];
    open (SHELL, "| /bin/sh -x");
    print SHELL $commands;
    close (SHELL);
    return $?;
}   ##run_shell

############################################################################
sub inst_perl
############################################################################
{
    my $dist_name = $_[0];
    return run_shell(<< "COMMANDS");
cd ${source_dir}/${dist_name}
perl Makefile.PL INSTALLPRIVLIB=$lib INSTALLARCHLIB=$lib INSTALLSITELIB=$lib INSTALLSITEARCH=$lib PREFIX=$pref
make
make install
COMMANDS
}   ##inst_perl

############################################################################
sub inst_libwww
############################################################################
{
    my $dist_name = $_[0];
    return run_shell(<< "COMMANDS");
cd ${source_dir}/${dist_name}
perl Makefile.PL -n INSTALLPRIVLIB=$lib INSTALLARCHLIB=$lib INSTALLSITELIB=$lib INSTALLSITEARCH=$lib PREFIX=$pref
make
make install
COMMANDS
}   ##inst_libwww

############################################################################
sub inst_expat
############################################################################
{
    my $dist_name = $_[0];
    return run_shell(<< "COMMANDS");
cd ${source_dir}/${dist_name}
./configure --enable-shared=no --prefix=$expat_home
make
make install
cd $expat_home
chmod -R a+w . 2>/dev/null
COMMANDS
}   ##inst_expat

############################################################################
sub inst_xmlpars
############################################################################
{
    my $dist_name = $_[0];
    return run_shell(<< "COMMANDS");
cd ${source_dir}/${dist_name}
perl Makefile.PL INSTALLPRIVLIB=$lib INSTALLARCHLIB=$lib INSTALLSITELIB=$lib INSTALLSITEARCH=$lib PREFIX=$pref EXPATINCPATH=$expat_inc EXPATLIBPATH=$expat_lib
make
make install
COMMANDS
}   ##inst_xmlpars

############################################################################
sub inst_xmldom     #22.02.03 22:45
############################################################################
{
    my $dist_name = $_[0];
    my $res = inst_perl($dist_name);
    if ( $res ) { return $res; }
    if ( $] < 5.006 ) {
        my $file = $lib.'/XML/DOM.pm';
        return run_shell(<< "COMMANDS");
patch $file - <<PATCHFILE
32d31
< use bytes;
PATCHFILE
COMMANDS
    }
    return 0;
}   ##inst_xmldom
